# stash_notifier_spec.rb
# Copyright 2013 Doug Ireton

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#     http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

require 'spec_helper'
require 'json'

describe StashNotifier do

  let(:build_result) do
    {
      state:       'successful',
      key:         'repo-master',
      name:        'repo-master-42',
      url:         'https://jenkins.example.com/view/All/job/myproject/57/console',
      description: 'This is the description'
    }
  end

  let(:args) do
    {
      stash_url:    'https://stash.example.com',
      stash_user:   'my_stash_user',
      stash_pwd:    'P@ssword',
      git_commit:   '68ccfc66971c2e1e2ca57e17707dbebb6c6b4ebb',
      build_status: build_result[:state],
      build_key:    build_result[:key],
      build_name:   build_result[:name],
      build_url:    build_result[:url],
      description:  build_result[:description],
      # you must supply http_client so that VCR can override it
      http_client:  Faraday.new
    }
  end

  let!(:stash_notifier) { StashNotifier.new(args) }

  it 'should have a SimVer compatible version number' do
    SIMVER_REGEXP = /^\d{1}\.\d{1,2}\.\d{1,2}\.*\w*$/
    expect(StashNotifier::VERSION).to match SIMVER_REGEXP
  end

  describe '#post_to_stash', :vcr do
    it 'returns 204 when passed valid parameters' do

      response = stash_notifier.post_to_stash
      expect(response.status).to eq 204
    end

    it 'returns 401 when passed an invalid stash_user' do
      stash_notifier.stash_user = 'bogus'

      expect { stash_notifier.post_to_stash }.to \
        raise_error(StashNotifier::Unauthorized,
                    'Authentication failed. Please check your credentials and try again.')
    end

    it 'returns 401 when passed an invalid stash_pwd' do
      stash_notifier.stash_pwd = 'bogus'

      expect { stash_notifier.post_to_stash }.to \
        raise_error(StashNotifier::Unauthorized,
                    'Authentication failed. Please check your credentials and try again.')
    end

    it 'returns 400 when passed an invalid git_commit' do
      stash_notifier.git_commit = 'bogus'

      expect { stash_notifier.post_to_stash }.to \
        raise_error(StashNotifier::BadRequest)
    end

    it 'returns 400 when the build state is missing or invalid' do
      build_result[:state] = 'bogus'
      stash_notifier.build_result = build_result.to_json

      expect { stash_notifier.post_to_stash }.to \
        raise_error(StashNotifier::BadRequest)
    end

    it 'returns 400 when the build key is missing or invalid' do
      build_result[:key] = nil
      stash_notifier.build_result = build_result.to_json

      expect { stash_notifier.post_to_stash }.to \
        raise_error(StashNotifier::BadRequest)
    end

    it 'returns 400 when the build url is missing or invalid' do
      build_result[:url] = 'bogus'
      stash_notifier.build_result = build_result.to_json

      expect { stash_notifier.post_to_stash }.to \
        raise_error(StashNotifier::BadRequest)
    end
  end

  describe '#to_s' do
    it 'returns a nice, descriptive string' do
      stash_notifier = StashNotifier.new(args)
      expect(stash_notifier.to_s).to start_with('StashNotifier connection')
    end

  end

  describe '#http_client' do
    it 'returns the http_client' do
      expect(stash_notifier.http_client).to be_an_instance_of(Faraday::Connection)
    end
  end

end
